@@ -17,7 +17,6 @@ import networkx as nx
 from abc import ABC, abstractmethod
 from .vector import HybridVectorStore
 from .llm import BaseLLMClient
-# Lazy import of ContextEngineer to avoid circular dependency
 
 logger = logging.getLogger(__name__)
 
@@ -310,48 +309,55 @@ class DynamicTaskGraph:
     
     async def _generate_adaptive_tasks(self, base_template: Dict, query_analysis: Dict[str, Any], context: ExecutionContext) -> List[Task]:
         tasks = []
-        complexity = query_analysis['estimated_processing_complexity']
+        task_id_counter = 0
+        task_id_map = {}  # Maps task_name to task_id
         
         # Process sequential tasks
         for task_name in base_template['tasks']:
-            task_config = self._get_adaptive_task_config(task_name, query_analysis, complexity)
-            task = self._create_task_instance(task_name, task_config, context)
+            task_id_counter += 1
+            task_id = f"{task_name}_{task_id_counter}"
+            task_id_map[task_name] = task_id
+            task_config = self._get_adaptive_task_config(task_name, query_analysis, context)
+            task = self._create_task_instance(task_name, task_config, context, task_id)
             if task:
                 # Add dependencies for tasks following parallel groups
-                if task_name in base_template.get('tasks', []) and 'parallel_groups' in base_template:
+                if 'parallel_groups' in base_template:
                     parallel_tasks = [t for group in base_template['parallel_groups'] for t in group]
                     if task_name not in parallel_tasks:
-                        # Find the index of the task
                         task_index = base_template['tasks'].index(task_name)
-                        # Add dependencies from the last parallel group before this task
                         for group in base_template['parallel_groups']:
-                            if base_template['tasks'].index(group[0]) < task_index:
+                            if group and base_template['tasks'].index(group[0]) < task_index:
                                 task.metadata.dependencies.update(
-                                    {f"{t}_{datetime.utcnow().timestamp()}" for t in group}
+                                    {task_id_map[t] for t in group if t in task_id_map}
                                 )
                 tasks.append(task)
+                logger.debug(f"Added task {task_id} with dependencies {task.metadata.dependencies}")
         
         # Process conditional tasks
         for task_name, condition in base_template.get('conditional_tasks', {}).items():
             if self._evaluate_task_condition(condition, query_analysis, context):
-                task_config = self._get_adaptive_task_config(task_name, query_analysis, complexity)
-                task = self._create_task_instance(task_name, task_config, context)
+                task_id_counter += 1
+                task_id = f"{task_name}_{task_id_counter}"
+                task_id_map[task_name] = task_id
+                task_config = self._get_adaptive_task_config(task_name, query_analysis, context)
+                task = self._create_task_instance(task_name, task_config, context, task_id)
                 if task:
                     tasks.append(task)
+                    logger.debug(f"Added conditional task {task_id} with dependencies {task.metadata.dependencies}")
         
         return tasks
     
-    def _get_adaptive_task_config(self, task_name: str, query_analysis: Dict, complexity: float) -> Dict:
+    def _get_adaptive_task_config(self, task_name: str, query_analysis: Dict, context: ExecutionContext) -> Dict:
         base_config = {
             'timeout_seconds': 30.0,
             'max_retries': 2,
             'priority': TaskPriority.MEDIUM
         }
-        if complexity > 2.0:
+        if query_analysis['estimated_processing_complexity'] > 2.0:
             base_config['timeout_seconds'] = 60.0
             base_config['max_retries'] = 3
             base_config['priority'] = TaskPriority.HIGH
-        elif complexity < 1.2:
+        elif query_analysis['estimated_processing_complexity'] < 1.2:
             base_config['timeout_seconds'] = 15.0
             base_config['max_retries'] = 1
         task_adaptations = {
@@ -389,9 +395,9 @@ class DynamicTaskGraph:
             base_config.update(task_adaptations[task_name])
         return base_config
     
-    def _create_task_instance(self, task_name: str, config: Dict, context: ExecutionContext) -> Optional[Task]:
+    def _create_task_instance(self, task_name: str, config: Dict, context: ExecutionContext, task_id: str) -> Optional[Task]:
         metadata = TaskMetadata(
-            task_id=f"{task_name}_{datetime.utcnow().timestamp()}",
+            task_id=task_id,
             name=task_name,
             description=config.get('description', f'Execute {task_name}'),
             priority=config.get('priority', TaskPriority.MEDIUM),
@@ -415,6 +421,7 @@ class DynamicTaskGraph:
         }
         task_class = task_classes.get(task_name)
         if task_class:
+            logger.debug(f"Creating task instance for {task_name} with ID {task_id}")
             return task_class(metadata)
         else:
             logger.warning(f"Unknown task type: {task_name}")
@@ -459,6 +466,8 @@ class AsyncTaskExecutor:
             if not ready_tasks:
                 remaining_tasks = [t for t in tasks if t.metadata.task_id not in completed_tasks]
                 logger.error(f"No ready tasks found. Remaining: {[t.metadata.task_id for t in remaining_tasks]}")
+                logger.debug(f"Completed tasks: {completed_tasks}")
+                logger.debug(f"All task dependencies: {[(t.metadata.task_id, t.metadata.dependencies) for t in tasks]}")
                 break
             
             parallel_tasks = []
@@ -518,6 +527,8 @@ class AsyncTaskExecutor:
             for dependency in task.metadata.dependencies:
                 if dependency in [t.metadata.task_id for t in tasks]:
                     graph.add_edge(dependency, task.metadata.task_id)
+                else:
+                    logger.warning(f"Unresolved dependency {dependency} for task {task.metadata.task_id}")
         return graph
     
     def get_execution_stats(self) -> Dict[str, Any]:
@@ -532,7 +543,7 @@ class AsyncTaskExecutor:
 class FetchDataTask(Task):
     async def execute(self, context: ExecutionContext) -> TaskResult:
         logger.info(f"Fetching data for query: {context.query}")
-        await asyncio.sleep(0.1)  # Simulate I/O delay
+        await asyncio.sleep(0.1)
         fetched_data = {
             'documents': [
                 {'content': f'Mock document 1 for: {context.query}', 'score': 0.9},
@@ -780,6 +791,7 @@ class GenerateResponseTask(Task):
 
 class AnalyzeDataTask(Task):
     async def execute(self, context: ExecutionContext) -> TaskResult:
+        logger.info(f"Analyzing data for query: {context.query}")
         await asyncio.sleep(0.3)
         analysis_result = {'analysis': 'Mock analytical insights', 'confidence': 0.8}
         return TaskResult(
@@ -791,6 +803,7 @@ class AnalyzeDataTask(Task):
 
 class GenerateInsightsTask(Task):
     async def execute(self, context: ExecutionContext) -> TaskResult:
+        logger.info(f"Generating insights for query: {context.query}")
         await asyncio.sleep(0.4)
         insights = {'insights': 'Generated insights based on analysis', 'actionable_items': 3}
         return TaskResult(
@@ -802,6 +815,7 @@ class GenerateInsightsTask(Task):
 
 class ComputeDifferencesTask(Task):
     async def execute(self, context: ExecutionContext) -> TaskResult:
+        logger.info(f"Computing differences for query: {context.query}")
         await asyncio.sleep(0.2)
         differences = {'differences': 'Computed differences between datasets', 'variance': 0.15}
         return TaskResult(
@@ -813,6 +827,7 @@ class ComputeDifferencesTask(Task):
 
 class IdentifyTrendsTask(Task):
     async def execute(self, context: ExecutionContext) -> TaskResult:
+        logger.info(f"Identifying trends for query: {context.query}")
         await asyncio.sleep(0.3)
         trends = {'trends': 'Identified temporal trends', 'trend_strength': 0.7}
         return TaskResult(
@@ -824,6 +839,7 @@ class IdentifyTrendsTask(Task):
 
 class DetectAnomaliesTask(Task):
     async def execute(self, context: ExecutionContext) -> TaskResult:
+        logger.info(f"Detecting anomalies for query: {context.query}")
         await asyncio.sleep(0.25)
         anomalies = {'anomalies': 'Detected anomalies in data', 'anomaly_count': 2}
         return TaskResult(
@@ -861,6 +877,11 @@ class DAGEngine:
         logger.info(f"Executing workflow: {workflow_name} (type: {workflow_type})")
         
         try:
+            # Validate workflow before execution
+            validation = await self.validate_workflow(workflow_name)
+            if not validation['valid']:
+                raise Exception(f"Workflow validation failed: {validation['error']}")
+            
             tasks = await self.task_graph_generator.generate_dynamic_graph(workflow_type, initial_context)
             if not tasks:
                 raise Exception(f"No tasks generated for workflow: {workflow_name}")
@@ -982,6 +1003,12 @@ async def test_dag_engine():
         raw_data={'documents': []}
     )
     
+    # Validate workflow before execution
+    validation = await engine.validate_workflow('historical_analysis')
+    if not validation['valid']:
+        print(f"Workflow validation failed: {validation['error']}")
+        return
+    
     result_context = await engine.execute_workflow('historical_analysis', context)
     
     print("DAG Execution Results:")
